
#include "NetLoadModule.h"
#include "keywords.h"

#ifndef TRACE
#define TRACE(n) (void)0
#endif

/*

  !NetUninstallModule

  !NLMExecIf <module> [not] loaded|installed <command>

  {path}
  {name}
  {installed}
  {loaded}
  {doc}

  NetLoadModuleScripts <path>
  # path or zip to load install scripts from
  # when no script is included with the module
  
*/

void RegisterBangs(bangcmddef *Bangs)
{
	bangcmddef *pBang;
	char BangString[64];

	TRACE("RegisterBangs");

  BangString[0] = '!';

	pBang = Bangs;
	while (pBang->Name != NULL) {
		lstrcpy(BangString+1,pBang->Name);
		AddBangCommand(BangString, pBang->Command);
		pBang++;
	}
}

void RemoveBangs(bangcmddef *Bangs)
{
	bangcmddef *pBang;
	char BangString[64];

	TRACE("RemoveBangs");

  BangString[0] = '!';

	pBang = Bangs;
	while (pBang->Name != NULL) {
		lstrcpy(BangString+1,pBang->Name);
		RemoveBangCommand(BangString);
		pBang++;
	}
}


// ======================================================
//
// ======================================================


void BangNetReloadModule(HWND sender, LPCSTR args)
{
  _TCHAR buffer[NLM_PATH_LENGTH];
  /* reload yourself */ 
  if (GetToken(args, buffer, NULL, FALSE)
    && _tcscmp(buffer,".")==0)
  {
    SendMessage(GlobalData.hLiteStepWnd, LM_RELOADMODULE,
      (WPARAM)GlobalData.hInstance, LMM_HINSTANCE);
    return;
  }
  /* reload something else */ 
  s_Module *pMod = new s_Module; /* deleted by PML */ 
  ZeroMemory(pMod, sizeof(s_Module));

  if (GlobalData.pOpenBatchState)
    GlobalData.pOpenBatchState->bAnyFail = false;

  ParseModuleProc(pMod, args, PARSE_LOAD_MODULE);

  if (GlobalData.pOpenBatchState)
  {
    /* add it to the end of the list */ 
    s_Module **p = &GlobalData.pOpenBatchState->pModuleList;
    while (*p)
      p = &(*p)->pNext;
    *p = pMod;
  }
  else
  {
    ProcessModuleList(pMod);
  }
}

void BangNetUnloadModule(HWND sender, LPCSTR args)
{
  _TCHAR buffer[NLM_PATH_LENGTH+14];
  /* unload yourself */ 
  if (GetToken(args, buffer, NULL, FALSE)
    && _tcscmp(buffer,".")==0)
  {
    PostMessage(GlobalData.hLiteStepWnd, LM_UNLOADMODULE,
      (WPARAM)GlobalData.hInstance, LMM_HINSTANCE);
    return;
  }
  /* unload something else */ 
  _tcscpy(buffer, _T("!UnloadModule "));
  bool bWhoCaresIfItHasAnAlias;
  if (GetModulePath(args, _tcslen(args), buffer+14, NLM_PATH_LENGTH, bWhoCaresIfItHasAnAlias))
    LSExecute(sender, buffer, SW_SHOWNORMAL);
}

void BangNetInstallModule(HWND sender, LPCSTR args)
{
  s_Module *pMod = new s_Module;
  ZeroMemory(pMod, sizeof(s_Module));

  if (GlobalData.pOpenBatchState)
    GlobalData.pOpenBatchState->bAnyFail = false;

  ParseModuleProc(pMod, args, PARSE_INSTALL_MODULE);

  if (GlobalData.pOpenBatchState)
  {
    s_Module **p = &GlobalData.pOpenBatchState->pModuleList;
    while (*p)
      p = &(*p)->pNext;
    *p = pMod;
  }
  else
  {
    ProcessModuleList(pMod);
  }
}

static void UninstallPath(TCHAR *sDir, unsigned nDirLen,
                          TCHAR *sName, unsigned nNameLen)
{
  TCHAR file[NLM_PATH_LENGTH];
  WIN32_FIND_DATA f;
  HANDLE ffh;

  /* delete a subdirectory */ 
  _tcscpy(file, sDir);
  _tcscpy(file+nDirLen, sName);
  _tcscpy(file+nDirLen+nNameLen, _T("//*"));
  ZeroMemory(&f, sizeof(f));
  ffh = FindFirstFile(file, &f);
  if (ffh!=INVALID_HANDLE_VALUE)
  {
    do
    {
      _tcscpy(file+nDirLen+nNameLen+1, f.cFileName);
      DeleteFile(file);
    } while (FindNextFile(ffh, &f));
    FindClose(ffh);
    *(file+nDirLen+nNameLen) = 0;
    RemoveDirectory(file);
  }
  /* delete files */ 
  _tcscpy(file+nDirLen+nNameLen, _T(".*"));
  ZeroMemory(&f, sizeof(f));
  ffh = FindFirstFile(file, &f);
  if (ffh!=INVALID_HANDLE_VALUE)
  {
    do
    {
      _tcscpy(file+nDirLen, f.cFileName);
      /* must match filename with extension */ 
      /* ie, don't allow extra '.'s         */ 
      if (_tcschr(file+nDirLen+nNameLen+1,_T('.'))==NULL)
        DeleteFile(file);
    } while (FindNextFile(ffh, &f));
    FindClose(ffh);
  }
}

void BangNetUninstallModule(HWND sender, LPCSTR args)
{
  /* first, make sure it's not loaded */ 
  BangNetUnloadModule(sender, args);

  /* now remove it */ 
  _TCHAR token[NLM_PATH_LENGTH];
  if (GetToken(args, token, NULL, TRUE))
  {
    _TCHAR name[NLM_PATH_LENGTH];
    unsigned namelen;
    GetRawModuleName(name, token);
    namelen = _tcslen(name);

    UninstallPath(
      GlobalData.ModulePath, GlobalData.ModulePathLen,
      name, namelen);
    RemoveAlias(name);

    if (GlobalData.DocPath)
      UninstallPath(
        GlobalData.DocPath, GlobalData.DocPathLen,
        name, namelen);

    if (GlobalData.ZipPath)
      UninstallPath(
        GlobalData.ZipPath, GlobalData.ZipPathLen,
        name, namelen);
  }
}

void BangNetLoadModuleBatch(HWND sender, LPCSTR args)
{
  try
  {
    _TCHAR token[MAX_LINE_LENGTH];
    *token = 0;
    GetToken(args, token, NULL, FALSE);
    _tcslwr(token); /* make that case insensitive */ 
    switch (BatchKeywords[token])
    {
    case BATCH_BEGIN:
      /* if we're already in a batch, continue adding to it */ 
      if (!GlobalData.pOpenBatchState)
      {
        GlobalData.pOpenBatchState = new s_DownloadState;
        GlobalData.pOpenBatchState->pModuleList = NULL;
        GlobalData.pOpenBatchState->bAnyFail = false;
      }
      break;
    case BATCH_END:
      if (GlobalData.pOpenBatchState)
      {
        /* close the batch state first in case of nested batches */ 
        s_Module *pMod = GlobalData.pOpenBatchState->pModuleList;
        delete GlobalData.pOpenBatchState;
        GlobalData.pOpenBatchState = NULL;

        ProcessModuleList(pMod);
      }
      break;
    }
  }
  catch (...)
  {
		const TCHAR *args_[4];
    TCHAR buf[NLM_PATH_LENGTH];
		args_[0] = args;
		FormatMessage(
			FORMAT_MESSAGE_FROM_HMODULE|FORMAT_MESSAGE_ARGUMENT_ARRAY
      |FORMAT_MESSAGE_MAX_WIDTH_MASK,
			GlobalData.hInstance, NLM_ERROR_BATCH, 0,
			buf, NLM_PATH_LENGTH,
			(va_list *)args_);
    MessageBox(sender, buf, TITLE, ERROR_MESSAGE_FLAGS);
  }
}

// ======================================================
//
// ======================================================

bangcmddef Bangs[] =
{
	{ "NetReloadModule",	  &BangNetReloadModule },
	{ "NetUnloadModule",	  &BangNetUnloadModule },
  { "NetInstallModule",   &BangNetInstallModule },
  { "NetUninstallModule", &BangNetUninstallModule },
  { "NetLoadModuleBatch", &BangNetLoadModuleBatch },
	{ NULL, NULL }
};
